@@ -19,7 +19,8 @@
 #'        If `NA` (not recommended), output is _not_ intercepted.
 #'
 #' @param future.conditions A character string of conditions classes to be
-#'        captured and relayed.  The default is to relay messages and warnings.
+#'        captured and relayed.  The default is the same as the `condition`
+#'        argument of [future::Future()].
 #'        To not intercept conditions, use `conditions = character(0L)`.
 #'        Errors are always relayed.
 #'
@@ -122,6 +123,8 @@
 #' * `"random"`   - this will randomize the ordering via random index
 #'                  vector `sample.int(length(X))`.
 #' For example, `future.scheduling = structure(TRUE, ordering = "random")`.
+#' _Note_, when elements are processed out of order, then captured standard
+#' output and conditions are also relayed in that order, that is out of order.
 #'
 #' @example incl/future_lapply.R
 #'
@@ -129,13 +132,18 @@
 #'
 #' @importFrom globals globalsByName
 #' @importFrom future future resolve values as.FutureGlobals nbrOfWorkers getGlobalsAndPackages FutureError
-#' @importFrom utils capture.output head str
+#' @importFrom utils head str
 #' @export
-future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions = c("message", "warning"), future.globals = TRUE, future.packages = NULL, future.lazy = FALSE, future.seed = FALSE, future.scheduling = 1.0, future.chunk.size = NULL) {
+future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions = NULL, future.globals = TRUE, future.packages = NULL, future.lazy = FALSE, future.seed = FALSE, future.scheduling = 1.0, future.chunk.size = NULL) {
   stop_if_not(is.function(FUN))
   
   stop_if_not(is.logical(future.stdout), length(future.stdout) == 1L)
 
+  ## FIXME: Memoize the result
+  if (is.null(future.conditions)) {
+    future.conditions <- eval(formals(Future)[["conditions"]])
+  }
+  
   stop_if_not(is.logical(future.lazy), length(future.lazy) == 1L)
 
   stop_if_not(!is.null(future.seed))
@@ -197,12 +205,12 @@ future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions =
                        nbrOfWorkers = nbrOfWorkers(),
                        future.scheduling = future.scheduling,
                        future.chunk.size = future.chunk.size)
-  if (debug) mdebug("Number of chunks: %d", length(chunks))
+  if (debug) mdebugf("Number of chunks: %d", length(chunks))
 
   ## Process elements in a custom order?
   ordering <- attr(chunks, "ordering")
   if (!is.null(ordering)) {
-    if (debug) mdebug("Index remapping (attribute 'ordering'): [n = %d] %s", length(ordering), hpaste(ordering))
+    if (debug) mdebugf("Index remapping (attribute 'ordering'): [n = %d] %s", length(ordering), hpaste(ordering))
     chunks <- lapply(chunks, FUN = function(idxs) .subset(ordering, idxs))
   }
   
@@ -234,12 +242,12 @@ future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions =
   
   nchunks <- length(chunks)
   fs <- vector("list", length = nchunks)
-  if (debug) mdebug("Number of futures (= number of chunks): %d", nchunks)
+  if (debug) mdebugf("Number of futures (= number of chunks): %d", nchunks)
   
-  if (debug) mdebug("Launching %d futures (chunks) ...", nchunks)
+  if (debug) mdebugf("Launching %d futures (chunks) ...", nchunks)
   for (ii in seq_along(chunks)) {
     chunk <- chunks[[ii]]
-    if (debug) mdebug("Chunk #%d of %d ...", ii, length(chunks))
+    if (debug) mdebugf("Chunk #%d of %d ...", ii, length(chunks))
 
     X_ii <- X[chunk]
     globals_ii <- globals
@@ -248,7 +256,7 @@ future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions =
     packages_ii <- packages
 
     if (scanForGlobals) {
-      mdebug(" - Finding globals in 'X' for chunk #%d ...", ii)
+      mdebugf(" - Finding globals in 'X' for chunk #%d ...", ii)
       ## Search for globals in 'X_ii':
       gp <- getGlobalsAndPackages(X_ii, envir = envir, globals = TRUE)
       globals_X <- gp$globals
@@ -256,8 +264,8 @@ future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions =
       gp <- NULL
 
       if (debug) {
-        mdebug("   + globals found in 'X' for chunk #%d: [%d] %s", chunk, length(globals_X), hpaste(sQuote(names(globals_X))))
-        mdebug("   + needed namespaces for 'X' for chunk #%d: [%d] %s", chunk, length(packages_X), hpaste(sQuote(packages_X)))
+        mdebugf("   + globals found in 'X' for chunk #%d: [%d] %s", chunk, length(globals_X), hpaste(sQuote(names(globals_X))))
+        mdebugf("   + needed namespaces for 'X' for chunk #%d: [%d] %s", chunk, length(packages_X), hpaste(sQuote(packages_X)))
       }
     
       ## Export also globals found in 'X_ii'
@@ -275,7 +283,7 @@ future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions =
         if (length(packages_X) > 0L)
           packages_ii <- unique(c(packages_ii, packages_X))
       }
-      mdebug(" - Finding globals in 'X' for chunk #%d ... DONE", ii)
+      mdebugf(" - Finding globals in 'X' for chunk #%d ... DONE", ii)
     }
     
     X_ii <- NULL
@@ -289,7 +297,7 @@ future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions =
     if (length(chunk) > 1L) {
       globals_ii["...future.globals.maxSize"] <- list(globals.maxSize)
       options(future.globals.maxSize = length(chunk) * globals.maxSize.default)
-      if (debug) mdebug(" - Adjusted option 'future.globals.maxSize': %g -> %d * %g = %g (bytes)", globals.maxSize.default, length(chunk), globals.maxSize.default, getOption("future.globals.maxSize"))
+      if (debug) mdebugf(" - Adjusted option 'future.globals.maxSize': %g -> %d * %g = %g (bytes)", globals.maxSize.default, length(chunk), globals.maxSize.default, getOption("future.globals.maxSize"))
       on.exit(options(future.globals.maxSize = globals.maxSize), add = TRUE)
     }
     
@@ -312,7 +320,7 @@ future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions =
          globals = globals_ii, packages = packages_ii,
          lazy = future.lazy)
     } else {
-      if (debug) mdebug(" - seeds: [%d] <seeds>", length(chunk))
+      if (debug) mdebugf(" - seeds: [%d] <seeds>", length(chunk))
       globals_ii[["...future.seeds_ii"]] <- seeds[chunk]
       fs[[ii]] <- future({
         ...future.globals.maxSize.org <- getOption("future.globals.maxSize")
@@ -335,37 +343,37 @@ future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions =
     ## Not needed anymore
     rm(list = c("chunk", "globals_ii"))
 
-    if (debug) mdebug("Chunk #%d of %d ... DONE", ii, nchunks)
+    if (debug) mdebugf("Chunk #%d of %d ... DONE", ii, nchunks)
   } ## for (ii ...)
-  if (debug) mdebug("Launching %d futures (chunks) ... DONE", nchunks)
+  if (debug) mdebugf("Launching %d futures (chunks) ... DONE", nchunks)
 
   ## Not needed anymore
   rm(list = c("chunks", "globals", "envir"))
 
 
   ## 4. Resolving futures
-  if (debug) mdebug("Resolving %d futures (chunks) ...", nchunks)
+  if (debug) mdebugf("Resolving %d futures (chunks) ...", nchunks)
   
   values <- values(fs)
   ## Not needed anymore
   rm(list = "fs")
 
   if (debug) {
-    mdebug(" - Number of value chunks collected: %d", length(values))
-    mdebug("Resolving %d futures (chunks) ... DONE", nchunks)
+    mdebugf(" - Number of value chunks collected: %d", length(values))
+    mdebugf("Resolving %d futures (chunks) ... DONE", nchunks)
   }
 
   ## Sanity check
   stop_if_not(length(values) == nchunks)
   
-  if (debug) mdebug("Reducing values from %d chunks ...", nchunks)
+  if (debug) mdebugf("Reducing values from %d chunks ...", nchunks)
    
   values2 <- do.call(c, args = values)
   
   if (debug) {
-    mdebug(" - Number of values collected after concatenation: %d",
+    mdebugf(" - Number of values collected after concatenation: %d",
            length(values2))
-    mdebug(" - Number of values expected: %d", nX)
+    mdebugf(" - Number of values expected: %d", nX)
   }
 
   assert_values2(nX, values, values2, fcn = "future_lapply()", debug = debug)
@@ -381,14 +389,14 @@ future_lapply <- function(X, FUN, ..., future.stdout = TRUE, future.conditions =
     idx <- 1:nX
     invOrdering[.subset(ordering, idx)] <- idx
     rm(list = c("ordering", "idx"))
-    if (debug) mdebug("Reverse index remapping (attribute 'ordering'): [n = %d] %s", length(invOrdering), hpaste(invOrdering))
+    if (debug) mdebugf("Reverse index remapping (attribute 'ordering'): [n = %d] %s", length(invOrdering), hpaste(invOrdering))
     values <- .subset(values, invOrdering)
     rm(list = c("invOrdering"))
   }
 
   names(values) <- names(X)
 
-  if (debug) mdebug("Reducing values from %d chunks ... DONE", nchunks)
+  if (debug) mdebugf("Reducing values from %d chunks ... DONE", nchunks)
   
   if (debug) mdebug("future_lapply() ... DONE")
   
